package de.uni_marburg.powersort.benchmark;

import de.uni_marburg.powersort.data.DataEnum;
import de.uni_marburg.powersort.sort.DummySort;
import de.uni_marburg.powersort.sort.MergeSort;
import de.uni_marburg.powersort.sort.TimSort;
import de.uni_marburg.powersort.data.ObjectSupplier;

import java.util.EnumSet;
import java.util.concurrent.TimeUnit;

/**
 * Custom benchmark.
 */
public class Main {
    public static void main(final String[] args) {
        final SortImpl[] sortImplementations = getSortImplementations();
        final EnumSet<DataEnum> dataEnums = getSortInputSuppliers();

        for (DataEnum dataEnum : dataEnums) {
            ObjectSupplier objectSupplier = dataEnum.get();
            System.out.println("\n" + objectSupplier.title());

            for (SortImpl sortImplementation : sortImplementations) {
                Object[] sortInput = objectSupplier.getCopy();

                final long startNanos = System.nanoTime();
                sortImplementation.sort(sortInput);
                final long stopNanos = System.nanoTime();

                final long durNanos = stopNanos - startNanos;
                final long durMillis = TimeUnit.NANOSECONDS.toMillis(durNanos);
                final String durFormatted = LongFormatter.formatUnderscore(durMillis);
                System.out.println(durFormatted + "," + sortImplementation.title);
            }
        }
    }

    static SortImpl[] getSortImplementations() {
        return new SortImpl[]{
                new SortImpl("DummySort") {
                    @Override
                    public void sort(Object[] a) {
                        DummySort.sort(a, 0, a.length, NaturalOrder.INSTANCE, null, 0, 0);
                    }
                },
                new SortImpl("TimSort") {
                    @Override
                    public void sort(Object[] a) {
                        TimSort.sort(a, 0, a.length, NaturalOrder.INSTANCE, null, 0, 0);
                    }
                },
//                new SortImpl("FinnSort") {
//                    @Override
//                    public void sort(Object[] a) {
//                        FinnSort.sort(a);
//                    }
//                },
                new SortImpl("MergeSort") {
                    @Override
                    public void sort(Object[] a) {
                        MergeSort.legacyMergeSort(a, NaturalOrder.INSTANCE);
                    }
                }
        };
    }

    /**
     * The returned ObjectSupplier objects are wrapped by DataEnum objects.
     * This way they are lazily created on their first access with DataEnum.get().
     * This saves memory if we work with large lists.
     */
    static EnumSet<DataEnum> getSortInputSuppliers() {
        return EnumSet.allOf(DataEnum.class);
    }
}
